use crate::{
    schema::{AssocData, DatumDiscriminator, EntityVisitor},
    DBResult,
};

// trait implementations for Datum
mod datum_common;
// trait implementations for DatumList
mod datum_list;

// ----------------------------------------------------------------------
// Datum and related types
// ----------------------------------------------------------------------

/// Represents a data field in an Entity.
pub trait Datum: 'static {
    fn sql_type() -> &'static str;

    fn bind_to<'a>(&self, _stmt: &mut sqlite::Statement<'a>, index: usize);
    fn build_from<'a>(
        adata: AssocData,
        stmt: &mut sqlite::Statement<'a>,
        index: usize,
    ) -> DBResult<(Self, usize)>
    where
        Self: Sized;

    fn accept_entity_visitor(_: &mut impl EntityVisitor) {}
    fn accept_discriminator(d: &mut impl DatumDiscriminator)
    where
        Self: Sized,
    {
        d.visit_bare_field::<Self>();
    }
}

/// A fixed-length list of EntityDatums, usually a tuple.
pub trait DatumList {
    fn accept(&self, visitor: &mut impl DatumVisitor);
}

/// A walker for a DatumList instance.
pub trait DatumVisitor {
    fn visit<ED: Datum>(&mut self, datum: &ED);
}
